% Script to compute state-feedback gains for a single-input single-output 
% third-order system, with possibly an additional integral state.
%
% Input: matrices A(3x3), B(3x1), C(1x3) and D(1x1) for a system described by
%     d/dt x(t) = Ax(t) + Bu(t), y(t) = Cx(t) + Du(t)%
% 
% Output: K(1x3) and Kr, state gains and input scaling or
%         K(1x4) where K(4) is the gain on the integral state.
%
% K's are determined using pole placement or LQR

% NOTE: Find and fix anywhere it says FIX THIS!!

close all; % close all the plots.
clearvars;

% A matrix, multiplies x
A = zeros(3);
% B matrix multiplies the scalar input u
B = zeros(3,1);
% C matrix, multiplies x
C = zeros(1,3);
% D matrix multiplies u, scalar, no preallocation.

% Fill in the A and B matrix entries

%********FIX THIS FIRST!!!!!!**********************
% First Row
A(1,1) = 0.0;
A(1,2) = 0.0;
A(1,3) = 0.0;
B(1) = 0.0;
s1 = 'icoil';

%********FIX THIS FIRST!!!!!!**********************
% Second Row
A(2,1) = 0.0;
A(2,2) = 0.0;
A(2,3) = 0.0;
B(2) = 0.0;
s2 = 'vel';

%********FIX THIS FIRST!!!!!!**********************
% Third Row
A(3,1) = 0.0;
A(3,2) = 0.0;
A(3,3) = 0.0;
B(3) = 0.0;
s3 = 'y';

%********FIX THIS FIRST!!!!!!**********************
% Fill in the output matrix
C(1) = 0.0;
C(2) = 0.0;
C(3) = 0.0;

% The instantaneous u-to-y term (D) is zero.
D = 0;

% Convert to a system data structure
sys = ss(A, B, C, D, 'StateName', {s1,s2,s3});

% Print input-output transfer function.
tfsys = tf(sys);
if tfsys.num{1}(end) ~= 0
    fprintf('Open loop system')
    display(tfsys);
end

% Determine poles
ps = pole(sys);
fprintf('Open-Loop Poles:%f,%f,%f\n', ps(1),ps(2),ps(3));

% Select good closed loop poles for pole placement design
% *********FIX THIS SECOND!!!**************
% poles = [-1000, -2000, -3000];
% Kplace = place(sys.A,sys.B,poles);

% Select state weights Q, and control weights R 
% *********FIX THIS SECOND!!!**************
% Q = diag([1,1,1]); % Metric: Q(1,1)*x1^2+Q(2,2)*x2^2+Q(3,3)*x3^2 + R*u^2
% R = 1;
% Klqr = lqr(sys,Q,R);

% Select Kplace or Klqr 
%*********FIX THIS SECOND!!!**************
%K = Klqr; fprintf('lqr controller selected\n')
%K = Kplace; fprintf('pole placement controller selected\n')

% Compute input scaling Kr so that feedback u = Kr*y_d - K*x
%   y_d  - y = 0 in steady-state.
%********FIX THIS SECOND!!!!!!**********************
%Kr = 0.0;

%*********FIX THIS FIRST!!!**************
% Pick the K's and Kr for PD controller
K = [0.0 0.0 0];
Kr = 0.0;

% Compute matrices Acl and Bcl for the closed loop to have equation
%    dx/dt = Acl*x + Bcl*u
%********FIX THIS FIRST!!!!!!**********************
Acl = A;  
Bcl = B;

% Recall that u = Kr*y_d - K*x for the open loop system, and that for the 
% closed-loop system described by Acl,Bcl,Ccl,Dcl matrices, 
% dx/dt = Acl*x + Bcl*y_d and y = Ccl*x + Dcl*y_d

%********FIX THIS FIRST!!!!!!**********************
% The Cplot and Dplot matrices below are set so that 
% the second through fourth elements of Cplot*x + Dplot*y_d are the three 
% states. Replace the zero elements in Cplot and Dplot below so that the 
% first element of Cplot*x + Dplot*y_d equals the command,
% u =  = Kr*y_d - K*x
Cplot = [0.0, 0.0, 0.0; eye(3)]; %****FIX THIS FIRST!!!!!!************
Dplot = [0.0; zeros(3,1);]; %****FIX THIS FIRST!!!!!!************

% Create the closed loop systen
sys_cl = ss(Acl, Bcl, Cplot, Dplot, ...
    'OutputName', {'cmd', s1, s2, s3});
ps = pole(sys_cl);
fprintf('Closed-Loop Poles:%f,%f,%f\n', ps(1),ps(2),ps(3));

% Plot step response of the command and all three states 
figure;
step(sys_cl, 1.0); grid
fprintf('K%s=%f, K%s=%f, K%s=%f, Kr=%f\n', s1,K(1),s2,K(2),s3,K(3), Kr);


fprintf('\nStarting Controller with Integrator\n');
% *********FIX THIS THIRD!!!**************

% to add the integrator part to the controller, we augment the original
% equations dx/dt=Ax+Bu by the "integrator state" w satisfying
% dw/dr = y - y_d . The resulting "complete state" xp=[w;x]
% satisfies dxp/dt=Ap*xp+Bp*u+Br*y_d
% The "p" is to indicate matrices for controller with an integral
% That is, Ap is 4x4, Bp is 4x1, Cp is 1x4.
% Please determine what to put in the 1st row and column.
Bp = [0; B];
Cp = [0 C];
Dp = D;
Ap = zeros(4);
Ap(2:4,:) = [zeros(3,1) A]; % we set the first 3 rows of Ap for you

% ******** define the first row of Ap
% *********FIX THIS THIRD!**** 
Ap(1, :) = [0 0 0 0];   %*********FIX THIS THIRD!****

% Convert to a system (lqr design ignores Br)
sysp = ss(Ap, Bp, Cp, Dp, 'StateName',{'Int', s1, s2, s3});
Kp = [1,0,0,0];

% Select state weights Q, and control weights R 
% State weights*********FIX THIS THIRD! *********
Qp = diag([1,1,1,1]);
Rp = diag(1);
%Kp = lqr(sysp,Qp,Rp);

% with u=-Kp*xp, the closed-loop system has the form
%    dxp/dt = Aclp*xp + Bclp*y_d
Aclp = Ap;  % *********FIX THIS THIRD!****
Bclp = [0;0;0;0];   %*********FIX THIS THIRD!****

% [xp;u] = Cplotp*xp+Dplotp*y_d
Cplotp = [zeros(1,4); eye(4);]; % *********FIX THIS THIRD!****
Dplotp = [0.0; zeros(4,1)]; 

% Closed Loop systen
sys_clp = ss(Aclp, Bclp, Cplotp, Dplotp, ...
'OutputName', {'cmd',s1,s2,s3,'Integral'});
ps = pole(sys_clp);
fprintf('Closed-Loop Poles:%f,%f,%f\n', ps(1),ps(2),ps(3));


% Plot step response
figure;
step(sys_clp,1); grid
fprintf('Kint=%f K%s=%f, K%s=%f, K%s=%f, \n',Kp(1),s1,Kp(2),s2,Kp(3),s3,Kp(4));

fprintf('Done With Integrator\n');
%FIX THIS: remove the above line when adding the integral term.
%}
